require 'monitors/violations_recorders/base'
require 'timeout'

module Monitors
  module ViolationsRecorders
    class ResponseTime < Base
      class << self
        def retries_limit
          Cfg.response_time_check_retries
        end

        def process_attr
          :pid
        end

        def process_is_violating?(process)
          time = Cfg.response_time_threshold 
          phrase = nil
          begin 
            Timeout.timeout(Cfg.response_time_threshold) do 
              response = get_response(process)
              time = response.split("***").last.to_f
              phrase = response.match(Cfg.response_check_phrase)
            end
          rescue Timeout::Error
            return true
          end
          ( time > Cfg.response_time_threshold ) || !(phrase)
        end

        private

        def get_response(process)
          `curl -s -w "***%{time_total}" --header "X-Backend-Host: #{Cfg.response_time_check_host}" --header "X-Phantom: true" #{process_url(process)}`
        end

        def process_url(process)
          "http://localhost:#{process.port}#{Cfg.response_time_check_path}"
        end
      end
    end
  end
end
